@@ -30,10 +30,10 @@
 #include "gossip-utils.h"
 #include "gossip-account-manager.h"
 
-#define RESOURCE_HACK /* For splitting an id of user@server/resource
-		       * to just user@server with resource in it's own
-		       * xml tags (for Gossip release 0.10.2).
-		       */
+/* For splitting an id of user@server/resource to just user@server with resource
+ * in it's own xml tags (for Gossip release 0.10.2).
+ */
+#define RESOURCE_HACK
 
 #define ACCOUNTS_XML_FILENAME "accounts.xml"
 #define ACCOUNTS_DTD_FILENAME "gossip-account.dtd"
@@ -133,6 +133,8 @@ account_manager_finalize (GObject *object)
 
 	g_free (priv->default_name);
 	g_free (priv->default_name_override);
+
+	G_OBJECT_CLASS (gossip_account_manager_parent_class)->finalize (object);
 }
 
 GossipAccountManager *
@@ -368,7 +370,9 @@ gossip_account_manager_get_default (GossipAccountManager *manager)
 			GList         *l;
 
 			l = gossip_account_manager_get_accounts (manager);
-			account = g_list_nth_data (l, 0);
+			account = g_object_ref (l->data);
+
+			g_list_foreach (l, (GFunc) g_object_unref, NULL);
 			g_list_free (l);
 
 			name = gossip_account_get_name (account);
@@ -749,6 +753,9 @@ account_manager_file_save (GossipAccountManager *manager)
 
 	g_free (xml_file);
 
+	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
+	g_list_free (accounts);
+	
 	return TRUE;
 }
 
