@@ -101,8 +101,6 @@ enum {
 
 static guint chat_signals[LAST_SIGNAL] = { 0 };
 
-static GObjectClass *parent_class = NULL;
-
 G_DEFINE_TYPE (GossipChat, gossip_chat, G_TYPE_OBJECT);
 
 static void
@@ -111,7 +109,6 @@ gossip_chat_class_init (GossipChatClass *klass)
 	GObjectClass *object_class;
 
 	object_class = G_OBJECT_CLASS (klass);
-	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = chat_finalize;
 
@@ -208,7 +205,7 @@ gossip_chat_init (GossipChat *chat)
 
 	/* get spelling languages */
 	value = gconf_client_get_string (gossip_app_get_gconf_client (),
-					 GCONF_PATH "/conversation/spell_checker_languages", 
+					 GCONF_CHAT_SPELL_CHECKER_LANGUAGES,
 					 NULL);
 	
 	if (value) {
@@ -248,10 +245,9 @@ chat_finalize (GObject *object)
 
 	if (priv->spell) {
 		gossip_spell_unref (priv->spell);
-		g_free (priv);
 	}
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (gossip_chat_parent_class)->finalize (object);
 }
 
 static void
