@@ -61,7 +61,7 @@ gossip_contact_groups_get_all (void)
 	gchar *dir;
 	gchar *file_with_path;
 
-	/* if already set up clean up first */
+	/* If already set up clean up first */
 	if (groups) {
 		g_list_foreach (groups, (GFunc)contact_group_free, NULL);
 		g_list_free (groups);
@@ -70,7 +70,7 @@ gossip_contact_groups_get_all (void)
 
 	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
 	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		mkdir (dir, S_IRUSR | S_IWUSR | S_IXUSR);
+		g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	}
 
 	file_with_path = g_build_filename (dir, CONTACT_GROUPS_XML_FILENAME, NULL);
@@ -104,7 +104,7 @@ contact_groups_file_parse (const gchar *filename)
 		return;
 	}
 
-	if (!gossip_utils_xml_validate (doc, CONTACT_GROUPS_DTD_FILENAME)) {
+	if (!gossip_xml_validate (doc, CONTACT_GROUPS_DTD_FILENAME)) {
 		g_warning ("Failed to validate file:'%s'", filename);
 		xmlFreeDoc(doc);
 		xmlFreeParserCtxt (ctxt);
@@ -194,16 +194,16 @@ contact_groups_file_save (void)
 	xmlNodePtr  node;
 	GList      *l;
 	gchar      *dtd_file;
-	gchar      *xml_dir;
-	gchar      *xml_file;
+	gchar      *dir;
+	gchar      *file;
 
-	xml_dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
-	if (!g_file_test (xml_dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
-		mkdir (xml_dir, S_IRUSR | S_IWUSR | S_IXUSR);
+	dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
+	if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
+		g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	}
 
-	xml_file = g_build_filename (xml_dir, CONTACT_GROUPS_XML_FILENAME, NULL);
-	g_free (xml_dir);
+	file = g_build_filename (dir, CONTACT_GROUPS_XML_FILENAME, NULL);
+	g_free (dir);
 
 	dtd_file = g_build_filename (DTDDIR, CONTACT_GROUPS_DTD_FILENAME, NULL);
 
@@ -227,15 +227,15 @@ contact_groups_file_save (void)
 		xmlNewProp (subnode, "name", cg->name);
 	}
 
-	DEBUG_MSG (("ContactGroups: Saving file:'%s'", xml_file));
-	xmlSaveFormatFileEnc (xml_file, doc, "utf-8", 1);
+	DEBUG_MSG (("ContactGroups: Saving file:'%s'", file));
+	xmlSaveFormatFileEnc (file, doc, "utf-8", 1);
 	xmlFreeDoc (doc);
 
 	xmlCleanupParser ();
 
 	xmlMemoryDump ();
 	
-	g_free (xml_file);
+	g_free (file);
 
 	return TRUE;
 }
